import os
import sys
import xbmcgui
import xbmcplugin
import xbmcaddon
import xbmcvfs
import resolveurl as resolver
import json
from time import time
from datetime import datetime
from urllib.parse import parse_qsl
from urllib.parse import urlparse
from resources.lib.duckduckgo_search import ddg_videos
from resources.lib.SearchHistory.search_history import SearchHistory
from resources.lib.SearchHistory.vfs import VFS
import libardnewjsonparser

class Multisearch:
    def __init__(self,url,handle):
        self.url = url
        self.handle = handle
        self.addon = xbmcaddon.Addon(os.environ.get('ADDON_ID', ''))
        self.addon_profile = xbmc.translatePath(self.addon.getAddonInfo('profile'))
        self.region = self.addon.getSetting("region")
        self.show_platform_dialog = self.addon.getSetting("show_platform_dialog") == "true"
        self.working_path = self.addon_profile
        if not xbmcvfs.exists(self.working_path): xbmcvfs.mkdirs(self.working_path)
        self.vfs = VFS(self.addon_profile)
        self.search_history = SearchHistory('history.json',self.vfs)
        self.ard_domains = ["www.ardmediathek.de"]
        self.zdf_domains = ["www.zdf.de"]
        self.valid_domains = ["www.ardmediathek.de", "www.youtube.com", "vimeo.com", "www.dailymotion.com"]
        
    def entry(self):
        listing = []
        list_item = xbmcgui.ListItem(label = self.addon.getLocalizedString(30050))
        list_item.setProperty('IsPlayable', 'false')
        url = '{0}?action=newsearch'.format(self.url)
        is_folder = True
        listing.append((url, list_item, is_folder))
        xbmcplugin.addDirectoryItems(self.handle, listing, len(listing))
        
        listing = []
        list_item = xbmcgui.ListItem(label = self.addon.getLocalizedString(30057))
        list_item.setProperty('IsPlayable', 'false')
        url = self.url
        is_folder = True
        listing.append((url, list_item, is_folder))
        xbmcplugin.addDirectoryItems(self.handle, listing, len(listing))

        history = self.search_history.get()
        for k in list(history):
            listing = []
            query = history[k].get("query")
            list_item = xbmcgui.ListItem(label=query)
            list_item.setProperty('IsPlayable', 'false')
            list_item.addContextMenuItems([(self.addon.getLocalizedString(30052), "RunPlugin({0}?action=remove&query={1})".format(self.url, query)),])
            url = '{0}?action=search&query={1}'.format(self.url, query)
            is_folder = True
            listing.append((url, list_item, is_folder))
            xbmcplugin.addDirectoryItems(self.handle, listing, len(listing))

        xbmcplugin.endOfDirectory(self.handle)
        
    def play(self, url):
        domain = urlparse(url).netloc
        resolved = None
        
        if domain in self.ard_domains:
            rev_url = url[::-1]
            pos = len(url) - rev_url.index("/")
            if pos>0:
                videoid = url[pos:]
                o = libardnewjsonparser.parser()
                mediainfo = o.parseVideo(videoid)
                resolved = mediainfo["media"][0]["url"]
        else:
            try:
                resolved = resolver.resolve(url)
            except:
                pass
                
        if resolved:
            play_item = xbmcgui.ListItem(path=resolved)
            xbmcplugin.setResolvedUrl(self.handle, True, listitem=play_item)
            
        
    def showresults(self, query, page):
        videos = ddg_videos(query, region=self.region, page=int(page))
        page = str(int(page)+1)
        for video in videos:
            s = video['content']
            d = urlparse(s).netloc 
            if d in self.valid_domains:
                iconurl = "http://" + d + "/favicon.ico"
    
                listing = []
                list_item = xbmcgui.ListItem(label=video['title'])            
                list_item.setProperty('IsPlayable', 'true')
                list_item.setArt({'icon' : iconurl, 'poster' : video['images']['large'] })
                list_item.setInfo('video', { 'plot': video['description'] })
                list_item.setInfo('video', { 'duration': video['duration'] })
                url = '{0}?action=play&url={1}'.format(self.url, s)
                is_folder = False
                listing.append((url, list_item, is_folder))
                xbmcplugin.addDirectoryItems(self.handle, listing, len(listing))
            
        if len(videos)>50:
            listing = []
            list_item = xbmcgui.ListItem(label = self.addon.getLocalizedString(30055))
            list_item.setProperty('IsPlayable', 'false')
            url = '{0}?action=showresults&query={1}&page={2}'.format(self.url, query, page)
            is_folder = True
            listing.append((url, list_item, is_folder))
            xbmcplugin.addDirectoryItems(self.handle, listing, len(listing))
        
        xbmcplugin.endOfDirectory(self.handle)
        
    def search(self, query):
        if self.show_platform_dialog:
            li0 = xbmcgui.ListItem(label = self.addon.getLocalizedString(30004))
            li1 = xbmcgui.ListItem(label = self.addon.getLocalizedString(30005))
            li1.setArt({'icon' : 'https://www.youtube.com/favicon.ico'})
            li2 = xbmcgui.ListItem(label = self.addon.getLocalizedString(30006))
            li2.setArt({'icon' : 'https://vimeo.com/favicon.ico'})
            li3 = xbmcgui.ListItem(label = self.addon.getLocalizedString(30007))
            li3.setArt({'icon' : 'https://www.dailymotion.com/favicon.ico'})
            li4 = xbmcgui.ListItem(label = self.addon.getLocalizedString(30009))
            li4.setArt({'icon' : 'https://www.ardmediathek.de/favicon.ico'})
#            li5 = xbmcgui.ListItem(label = self.addon.getLocalizedString(30010))
#            li5.setArt({'icon' : 'https://www.zdf.de/favicon.ico'})

            hoster = [li0, li1, li2, li3, li4]
            selected_platform = xbmcgui.Dialog().select(self.addon.getLocalizedString(30003), hoster, useDetails=True)
        
            if selected_platform == 0: pass
            elif selected_platform == 1: query += " site:youtube.com"
            elif selected_platform == 2: query += " site:vimeo.com"
            elif selected_platform == 3: query += " site:dailymotion.com"
            elif selected_platform == 4: query += " site:ardmediathek.de"
#            elif selected_platform == 5: query += " site:zdf.de"        
            else: self.entry()
            self.showresults(query,1)
        else:
            selected_platform = self.addon.getSetting("selected_platform")
            if selected_platform == self.addon.getLocalizedString(30005): query += " site:youtube.com"
            elif selected_platform == self.addon.getLocalizedString(30006): query += " site:vimeo.com"
            elif selected_platform == self.addon.getLocalizedString(30007): query += " site:dailymotion.com"
            elif selected_platform == self.addon.getLocalizedString(30009): query += " site:ardmediathek.de"
 #           elif selected_platform == self.addon.getLocalizedString(30010): query += " site:zdf.de"
            self.showresults(query,1)
        
    def newsearch(self):
        query = xbmcgui.Dialog().input(self.addon.getLocalizedString(30050))

        if query != "" :
            self.search_history.add(query)
            self.search(query)
        else:
            self.entry() 
        
    def removequery(self, query):
        self.search_history.remove(query)
        xbmc.executebuiltin("Container.Refresh")
        
    def router(self, paramstring):
        params = dict(parse_qsl(paramstring[1:]))
        if params:
            if params['action'] == 'newsearch':
                self.newsearch()
            elif params['action'] == 'search':
                self.search(params['query'])
            elif params['action'] == 'showresults':
                self.showresults(params['query'],params['page'])
            elif params['action'] == 'play':
                self.play(params['url'])
            elif params['action'] == 'remove':
                self.removequery(params['query'])
            elif params['action'] == 'startsearch':
                self.start_search()
        else:
            self.entry()
                        
if __name__ == '__main__':
    ms = Multisearch(sys.argv[0], int(sys.argv[1]))
    ms.router(sys.argv[2])












